from .copy import copy_dir
from .delete import delete_dir
from .exceptions import S3Error
from .upload import (
    ObjectManager,
    create_dir_redirect_object,
    upload_dir,
    upload_file,
    upload_object,
)
from .utils import open_bucket



"""Tests for s3delete.

These tests required the following environment variables:

``LTD_TEST_AWS_ID``
    AWS access key ID

``LTD_TEST_AWS_SECRET``
    AWS secret access key

``LTD_TEST_BUCKET``
    Name of an S3 bucket that already exists and can be used for testing.

The tests will be skipped if they are not available.

Note that this test will create a random uuid4) directory at the root of
``LTD_TEST_BUCKET``, though the test harness will attempt to delete
it.
"""

from __future__ import annotations

import os
import uuid
from typing import TYPE_CHECKING

import boto3
import pytest

from ltdconveyor.s3 import delete_dir
from ltdconveyor.testutils import upload_test_files

if TYPE_CHECKING:
    from _pytest.fixtures import FixtureRequest


@pytest.mark.skipif(
    os.getenv("LTD_TEST_AWS_ID") is None
    or os.getenv("LTD_TEST_AWS_SECRET") is None
    or os.getenv("LTD_TEST_BUCKET") is None,
    reason="Set LTD_TEST_AWS_ID, "
    "LTD_TEST_AWS_SECRET and "
    "LTD_TEST_BUCKET",
)
def test_delete_dir(request: FixtureRequest) -> None:
    test_bucket = os.getenv("LTD_TEST_BUCKET", "")
    aws_access_key_id = os.getenv("LTD_TEST_AWS_ID", "")
    aws_secret_access_key = os.getenv("LTD_TEST_AWS_SECRET", "")

    session = boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    s3 = session.resource("s3")
    bucket = s3.Bucket(os.getenv("LTD_TEST_BUCKET"))

    bucket_root = str(uuid.uuid4()) + "/"

    def cleanup() -> None:
        print("Cleaning up the bucket")
        delete_dir(
            test_bucket,
            bucket_root,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )

    request.addfinalizer(cleanup)

    file_paths = ["a/test1.txt", "a/b/test2.txt", "a/b/c/test3.txt"]
    upload_test_files(
        file_paths,
        bucket,
        bucket_root,
        "sample-key",
        "max-age=3600",
        "text/plain",
    )

    # Delete b/*
    delete_dir(
        test_bucket,
        bucket_root + "a/b/",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    # Ensure paths outside of that are still available, but paths in b/ are
    # deleted
    bucket_paths = []
    for obj in bucket.objects.filter(Prefix=bucket_root):
        if obj.key.endswith("/"):
            continue
        bucket_paths.append(obj.key)

    for p in file_paths:
        bucket_path = bucket_root + p
        if p.startswith("a/b"):
            assert bucket_path not in bucket_paths
        else:
            assert bucket_path in bucket_paths

    # Attempt to delete an empty prefix. Ensure it does not raise an exception.
    delete_dir(
        test_bucket,
        bucket_root + "empty-prefix/",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )



"""Login functionality for the LTD Keeper API."""

from urllib.parse import urljoin

import requests

from ltdconveyor.keeper.exceptions import KeeperError

__all__ = ["get_keeper_token"]


def get_keeper_token(host: str, username: str, password: str) -> str:
    """Get a temporary auth token from LTD Keeper.

    Parameters
    ----------
    host : `str`
        Hostname of the LTD Keeper API (e.g., ``'https://keeper.lsst.codes'``).
    username : `str`
        Username.
    password : `str`
        Password.

    Returns
    -------
    token : `str`
        LTD Keeper API token.

    Raises
    ------
    KeeperError
        Raised if the LTD Keeper API cannot return a token.
    """
    token_endpoint = urljoin(host, "/token")
    r = requests.get(token_endpoint, auth=(username, password))
    if r.status_code != 200:
        raise KeeperError(
            "Could not authenticate to {0}: error {1:d}\n{2}".format(
                host, r.status_code, r.json()
            )
        )
    return r.json()["token"]



"""Register and confirm new build uploads with the LTD Keeper API."""

import logging
from typing import Any, Dict, Optional, Sequence
from urllib.parse import urljoin

import requests
import uritemplate

from ltdconveyor.keeper.exceptions import KeeperError

__all__ = ["register_build", "confirm_build"]


def register_build(
    host: str,
    keeper_token: str,
    product: str,
    git_refs: Sequence[str],
    dirnames: Optional[Sequence[str]] = None,
) -> Dict[str, Any]:
    """Register a new build for a product on LSST the Docs.

    Wraps the ``POST /products/{product}/builds/`` **v2 API** endpoint.

    Parameters
    ----------
    host : `str`
        Hostname of LTD Keeper API server.
    keeper_token : `str`
        Auth token (`ltdconveyor.keeper.get_keeper_token`).
    product : `str`
        Name of the product in the LTD Keeper service.
    git_refs : `list` of `str`
        List of Git refs that correspond to the version of the build. Git refs
        can be tags or branches.
    dirnames : `list` of `str`
        A list of relative directory names in the site. Each directory will
        get its own presigned POST URL in the response from the LTD Keeper API.

    Returns
    -------
    build_info : `dict`
        LTD Keeper build resource.

    Raises
    ------
    ltdconveyor.keeper.KeeperError
        Raised if there is an error communicating with the LTD Keeper API.
    """
    logger = logging.getLogger(__name__)

    data = {"git_refs": git_refs}
    if dirnames is not None:
        data["directories"] = list(dirnames)

    endpoint_url = uritemplate.expand(
        urljoin(host, "/products/{p}/builds/"), p=product
    )

    r = requests.post(
        endpoint_url,
        auth=(keeper_token, ""),
        json=data,
        headers={"Accept": "application/vnd.ltdkeeper.v2+json"},
    )

    if r.status_code != 201:
        raise KeeperError(r.json())
    build_info: Dict[str, Any] = r.json()
    logger.debug("Registered a build for product %s:\n%s", product, build_info)
    return build_info


def confirm_build(build_url: str, keeper_token: str) -> None:
    """Confirm a build upload is complete.

    Wraps ``PATCH /builds/{build}``.

    Parameters
    ----------
    build_url : `str`
        URL of the build resource. Given a build resource, this URL is
        available from the ``self_url`` field.
    keeper_token : `str`
        Auth token (`ltdconveyor.keeper.get_keeper_token`).

    Raises
    ------
    ltdconveyor.keeper.KeeperError
        Raised if there is an error communicating with the LTD Keeper API.
    """
    data = {"uploaded": True}

    r = requests.patch(build_url, auth=(keeper_token, ""), json=data)
    if r.status_code != 200:
        raise KeeperError(r)



"""Command-line interface ``ltd`` as a Click application."""

import logging
from typing import Any, Optional

import click

from ltdconveyor.cli.upload import upload

__all__ = ["main"]

# Add -h as a help shortcut option
CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option(
    "--log-level",
    "log_level",
    type=click.Choice(["warning", "info", "debug"]),
    default="info",
    help="Logging level (for first-party messages). Default: `info`.",
)
@click.option(
    "--host",
    "keeper_hostname",
    default="https://keeper.lsst.codes",
    envvar="LTD_HOST",
    help="Hostname of the LTD Keeper API (or `$LTD_HOST`). "
    "Default: `https://keeper.lsst.codes`.",
)
@click.option(
    "-u",
    "--user",
    "username",
    envvar="LTD_USERNAME",
    help="Username for LTD Keeper (or `$LTD_USERNAME`).",
)
@click.option(
    "-p",
    "--password",
    "password",
    envvar="LTD_PASSWORD",
    help="Password for LTD Keeper (or `$LTD_PASSWORD`).",
)
@click.version_option()
@click.pass_context
def main(
    ctx: click.Context,
    log_level: str,
    keeper_hostname: str,
    username: str,
    password: str,
) -> None:
    """ltd is a command-line client for LSST the Docs.

    Use ltd to upload new site builds, and to work with the LTD Keeper API.
    """
    ch = logging.StreamHandler()
    if log_level in ("warning", "info"):
        formatter = logging.Formatter(
            "%(asctime)s %(levelname)8s | %(message)s"
        )
    else:
        formatter = logging.Formatter(
            "%(asctime)s %(levelname)8s %(name)s | %(message)s"
        )
    ch.setFormatter(formatter)

    logger = logging.getLogger("ltdconveyor")
    logger.addHandler(ch)
    logger.setLevel(log_level.upper())

    # Subcommands should use the click.pass_obj decorator to get this
    # ctx.obj object as the first argument.
    ctx.obj = {
        "keeper_hostname": keeper_hostname,
        "username": username,
        "password": password,
        "token": None,
    }


@main.command()
@click.argument("topic", default=None, required=False, nargs=1)
@click.pass_context
def help(ctx: click.Context, topic: Optional[str], **kw: Any) -> None:
    """Show help for any command."""
    # The help command implementation is taken from
    # https://www.burgundywall.com/post/having-click-help-subcommand
    if topic:
        if topic in main.commands:
            ctx.info_name = topic
            click.echo(main.commands[topic].get_help(ctx))
    else:
        assert ctx.parent
        click.echo(ctx.parent.get_help())


# Add subcommands from other modules
main.add_command(upload)



"""Helpers for the CLI (not click commands)."""

import logging
import sys

import click

from ltdconveyor.keeper.login import get_keeper_token

__all__ = ["ensure_login"]


def ensure_login(ctx: click.Context) -> None:
    """Ensure a token is in the Click context object or authenticate and obtain
    the token from LTD Keeper.

    Parameters
    ----------
    ctx : `click.Context`
        The Click context. ``ctx.obj`` must be a `dict` that contains keys:
        ``keeper_hostname``, ``username``, ``password``, ``token``. This
        context object is prepared by the main Click group,
        `ltdconveyor.cli.main.main`.
    """
    logger = logging.getLogger(__name__)

    if ctx.obj["token"] is None:
        if ctx.obj["username"] is None or ctx.obj["password"] is None:
            raise click.UsageError(
                "Use `ltd -u <username> -p <password> COMMAND` to "
                "authenticate to the LTD Keeper server."
            )
            sys.exit(1)

        logger.debug(
            "About to get token for user %s at %s",
            ctx.obj["username"],
            ctx.obj["keeper_hostname"],
        )

        token = get_keeper_token(
            ctx.obj["keeper_hostname"],
            ctx.obj["username"],
            ctx.obj["password"],
        )
        ctx.obj["token"] = token

        logger.debug(
            "Got token for user %s at %s",
            ctx.obj["username"],
            ctx.obj["keeper_hostname"],
        )

    else:
        logger.debug("Token already exists.")



"""Tests for s3copy.

These tests required the following environment variables:

``LTD_TEST_AWS_ID``
    AWS access key ID

``LTD_TEST_AWS_SECRET``
    AWS secret access key

``LTD_TEST_BUCKET``
    Name of an S3 bucket that already exists and can be used for testing.

The tests will be skipped if they are not available.

Note that this test will create a random uuid4) directory at the root of
``LTD_TEST_BUCKET``, though the test harness will attempt to delete
it.
"""

from __future__ import annotations

import os
import uuid
from typing import TYPE_CHECKING

import boto3
import pytest

from ltdconveyor.s3 import copy_dir, delete_dir
from ltdconveyor.testutils import upload_test_files

if TYPE_CHECKING:
    from _pytest.fixtures import FixtureRequest


@pytest.mark.skipif(
    os.getenv("LTD_TEST_AWS_ID") is None
    or os.getenv("LTD_TEST_AWS_SECRET") is None
    or os.getenv("LTD_TEST_BUCKET") is None,
    reason="Set LTD_TEST_AWS_ID, "
    "LTD_TEST_AWS_SECRET and "
    "LTD_TEST_BUCKET",
)
def test_copy_directory(request: FixtureRequest) -> None:
    test_bucket = os.getenv("LTD_TEST_BUCKET", "")
    aws_access_key_id = os.getenv("LTD_TEST_AWS_ID", "")
    aws_secret_access_key = os.getenv("LTD_TEST_AWS_SECRET", "")

    session = boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    s3 = session.resource("s3")
    bucket = s3.Bucket(os.getenv("LTD_TEST_BUCKET"))

    bucket_root = str(uuid.uuid4()) + "/"

    def cleanup() -> None:
        print("Cleaning up the bucket")
        delete_dir(
            test_bucket,
            bucket_root,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )

    request.addfinalizer(cleanup)

    initial_paths = ["test1.txt", "test2.txt", "aa/test3.txt"]
    new_paths = ["test4.txt", "bb/test5.txt"]

    # add old and new file sets
    upload_test_files(
        initial_paths,
        bucket,
        bucket_root + "a/",
        "sample-key",
        "max-age=3600",
        "text/plain",
    )
    upload_test_files(
        new_paths,
        bucket,
        bucket_root + "b/",
        "sample-key",
        "max-age=3600",
        "text/plain",
    )

    # copy files
    copy_dir(
        test_bucket,
        src_path=bucket_root + "b/",
        dest_path=bucket_root + "a/",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        surrogate_key="new-key",
        surrogate_control="max-age=31536000",
        cache_control="no-cache",
    )

    # Test files in the a/ directory are from b/
    for obj in bucket.objects.filter(Prefix=bucket_root + "a/"):
        bucket_path = os.path.relpath(obj.key, start=bucket_root + "a/")
        assert bucket_path in new_paths
        # ensure correct metadata
        head = s3.meta.client.head_object(Bucket=test_bucket, Key=obj.key)
        assert head["CacheControl"] == "no-cache"
        assert head["ContentType"] == "text/plain"
        assert head["Metadata"]["surrogate-key"] == "new-key"
        assert head["Metadata"]["surrogate-control"] == "max-age=31536000"

    # Test that a directory object exists
    bucket_paths = [
        obj.key for obj in bucket.objects.filter(Prefix=bucket_root + "a")
    ]
    assert os.path.join(bucket_root, "a") in bucket_paths


def test_copy_dir_src_in_dest() -> None:
    """Test that copy_directory fails raises a RuntimeError if source in
    destination.
    """
    with pytest.raises(RuntimeError):
        copy_dir(
            "example",
            "dest/src",
            "dest",
            aws_access_key_id="id",
            aws_secret_access_key="key",
        )


def test_copy_dir_dest_in_src() -> None:
    """Test that copy_directory fails raises a RuntimeError if destination
    is part of the source.
    """
    with pytest.raises(RuntimeError):
        copy_dir(
            "example",
            "src",
            "src/dest",
            aws_access_key_id="id",
            aws_secret_access_key="key",
        )



"""Tests for ltdconveyor.cli.upload."""

from typing import Any, List, Optional

import click
import pytest

from ltdconveyor.cli.upload import (
    _get_gh_actions_git_refs,
    _get_git_refs,
    _get_travis_git_refs,
    _should_skip_travis_event,
)


@pytest.mark.parametrize(
    "event_type,on_push,on_pr,on_api,on_cron,expected",
    [
        ("push", True, True, True, True, False),
        ("push", False, True, True, True, True),
        ("pull_request", True, True, True, True, False),
        ("pull_request", True, False, True, True, True),
        ("api", True, True, True, True, False),
        ("api", True, True, False, True, True),
        ("cron", True, True, True, True, False),
        ("cron", True, True, True, False, True),
    ],
)
def test_should_skip_travis_event(
    monkeypatch: Any,
    event_type: str,
    on_push: bool,
    on_pr: bool,
    on_api: bool,
    on_cron: bool,
    expected: bool,
) -> None:
    """Test _should_skip_travis_event with different combinations of
    user settings and TRAVIS_EVENT_TYPE variables.
    """
    import os

    def mock_env_var(_: Any) -> str:
        return event_type

    monkeypatch.setattr(os, "getenv", mock_env_var)
    assert (
        _should_skip_travis_event(on_push, on_pr, on_api, on_cron) is expected
    )


def test_get_travis_git_refs(monkeypatch: Any) -> None:
    import os

    monkeypatch.setattr(os, "getenv", lambda *args: "my_branch")

    assert _get_travis_git_refs() == ["my_branch"]


def test_get_travis_git_refs_no_env_var(monkeypatch: Any) -> None:
    import os

    monkeypatch.setattr(os, "getenv", lambda *args: "")

    with pytest.raises(click.UsageError):
        _get_travis_git_refs()


def test_get_gh_actions_git_refs(monkeypatch: Any) -> None:
    import os

    monkeypatch.setattr(os, "getenv", lambda *args: "refs/heads/my_branch")

    assert _get_gh_actions_git_refs() == ["my_branch"]


def test_get_gh_actions_git_refs_no_env_var(monkeypatch: Any) -> None:
    import os

    monkeypatch.setattr(os, "getenv", lambda *args: "")

    with pytest.raises(click.UsageError):
        _get_gh_actions_git_refs()


@pytest.mark.parametrize(
    "env_var,ci_env,user_git_ref,expected",
    [
        # using travis branch
        ("my_branch", "travis", None, ["my_branch"]),
        # overriding travis on command line
        ("my_branch", "travis", "user-branch", ["user-branch"]),
        # using GitHub actions branch
        ("refs/heads/my_branch", "gh", None, ["my_branch"]),
        ("refs/tags/my_tag", "gh", None, ["my_tag"]),
        ("refs/pull/my_pr", "gh", None, ["my_pr"]),
        # overriding GitHub actions on command line
        ("refs/heads/my_branch", "gh", "user-branch", ["user-branch"]),
        # only using command line arg
        ("my_branch", None, "user-branch", ["user-branch"]),
        (None, None, "user-branch", ["user-branch"]),
        # parsing user git ref list
        (None, None, "a b   c", ["a", "b", "c"]),
    ],
)
def test_get_git_refs(
    monkeypatch: Any,
    env_var: Optional[str],
    ci_env: Optional[str],
    user_git_ref: Optional[str],
    expected: List[str],
) -> None:
    import os

    monkeypatch.setattr(os, "getenv", lambda *args: env_var)

    assert _get_git_refs(ci_env, user_git_ref) == expected



"""ltd upload subcommand."""

import logging
import os
import re
import sys
from pathlib import Path
from typing import List, Optional

import click

from ltdconveyor.cli.utils import ensure_login
from ltdconveyor.keeper.build import confirm_build, register_build
from ltdconveyor.s3.presignedpost import (
    prescan_directory,
    upload_dir,
    upload_directory_objects,
)

__all__ = ["upload"]


@click.command()
@click.option("--product", required=True, help="Product name.")
@click.option(
    "--dir",
    "dirname",
    default=".",
    type=click.Path(exists=False, file_okay=False, dir_okay=True),
    help="Directory with files to upload. Default: `.` (current working "
    "directory).",
)
@click.option(
    "--git-ref",
    help="Git ref, or space-delimited list of git refs. This versions the "
    "build and helps LTD Keeper assign the build to an edition. "
    "Alternatively, version information can be auto-discovered by "
    "setting --travis in a Travis CI job or --gh for GitHub actions.",
)
@click.option(
    "--gh",
    "ci_env",
    flag_value="gh",
    help="Use environment variables from a GitHub Actions environment to set "
    "the --git-ref option.",
)
@click.option(
    "--travis",
    "ci_env",
    flag_value="travis",
    help="Use environment variables from a Travis CI environment to set "
    "the --git-ref option.",
)
@click.option(
    "--on-travis-push/--no-travis-push",
    "on_travis_push",
    default=True,
    help="Upload on a Travis CI push (enabled by default). Must be used with "
    "--travis.",
)
@click.option(
    "--on-travis-pr/--no-travis-pr",
    "on_travis_pr",
    default=False,
    help="Upload on a Travis CI pull request event (disabled by default). "
    "Must be used with --travis.",
)
@click.option(
    "--on-travis-api/--no-travis-api",
    "on_travis_api",
    default=True,
    help="Upload on a Travis CI API event (enabled by default). "
    "Must be used with --travis.",
)
@click.option(
    "--on-travis-cron/--no-travis-cron",
    "on_travis_cron",
    default=True,
    help="Upload on a Travis CI cron event (enabled by default). "
    "Must be used with --travis.",
)
@click.option(
    "--skip",
    "skip_upload",
    default=False,
    is_flag=True,
    envvar="LTD_SKIP_UPLOAD",
    help="Skip the upload, making the command a no-op. "
    "Useful in CI environments to disable a site upload just by setting "
    "this option or the environment variable $LTD_SKIP_UPLOAD=true.",
)
@click.pass_context
def upload(
    ctx: click.Context,
    product: str,
    git_ref: Optional[str],
    dirname: str,
    ci_env: str,
    on_travis_push: bool,
    on_travis_pr: bool,
    on_travis_api: bool,
    on_travis_cron: bool,
    skip_upload: bool,
) -> None:
    """Upload a new site build to LSST the Docs."""
    logger = logging.getLogger(__name__)

    if skip_upload:
        click.echo("Skipping ltd upload.")
        sys.exit(0)

    logger.debug("CI environment: %s", ci_env)
    logger.debug(
        "Travis events settings. " "On Push: %r, PR: %r, API: %r, Cron: %r",
        on_travis_push,
        on_travis_pr,
        on_travis_api,
        on_travis_cron,
    )

    # Abort upload on Travis CI under certain events
    if ci_env == "travis" and _should_skip_travis_event(
        on_travis_push, on_travis_pr, on_travis_api, on_travis_cron
    ):
        sys.exit(0)

    # Authenticate to LTD Keeper host
    ensure_login(ctx)

    # Detect git refs
    git_refs = _get_git_refs(ci_env, git_ref)

    # Prescan directory names. Each directory needs a presigned POST URL.
    base_dir = Path(dirname)
    dirnames = prescan_directory(base_dir)

    build_resource = register_build(
        ctx.obj["keeper_hostname"],
        ctx.obj["token"],
        product,
        git_refs,
        dirnames=dirnames,
    )
    logger.debug("Created build resource %r", build_resource)

    # Do the upload.
    upload_dir(post_urls=build_resource["post_prefix_urls"], base_dir=base_dir)
    logger.debug("Upload complete for %r", build_resource["self_url"])

    # Upload directory objects for redirects
    upload_directory_objects(post_urls=build_resource["post_dir_urls"],)

    # Confirm upload
    confirm_build(build_resource["self_url"], ctx.obj["token"])
    logger.info("Build %r complete", build_resource["self_url"])
    logger.info("Published build URL: %s", build_resource["published_url"])


def _should_skip_travis_event(
    on_travis_push: bool,
    on_travis_pr: bool,
    on_travis_api: bool,
    on_travis_cron: bool,
) -> bool:
    """Detect if the upload should be skipped based on the
    ``TRAVIS_EVENT_TYPE`` environment variable.

    Returns
    -------
    should_skip : `bool`
        True if the upload should be skipped based on the combination of
        ``TRAVIS_EVENT_TYPE`` and user settings.
    """
    travis_event = os.getenv("TRAVIS_EVENT_TYPE")
    if travis_event is None:
        raise click.UsageError(
            "Using --travis but the TRAVIS_EVENT_TYPE "
            "environment variable is not detected."
        )

    if travis_event == "push" and on_travis_push is False:
        click.echo("Skipping upload on Travis push event.")
        return True
    elif travis_event == "pull_request" and on_travis_pr is False:
        click.echo("Skipping upload on Travis pull request event.")
        return True
    elif travis_event == "api" and on_travis_api is False:
        click.echo("Skipping upload on Travis pull request event.")
        return True
    elif travis_event == "cron" and on_travis_cron is False:
        click.echo("Skipping upload on Travis cron event.")
        return True
    else:
        return False


def _get_git_refs(
    ci_env: Optional[str], user_git_ref: Optional[str]
) -> List[str]:
    if ci_env == "travis" and user_git_ref is None:
        # Get git refs from Travis environment
        git_refs = _get_travis_git_refs()
    elif ci_env == "gh" and user_git_ref is None:
        # Get git refs from GitHub Actions environment
        git_refs = _get_gh_actions_git_refs()
    elif user_git_ref is not None:
        # Get git refs from command line
        git_refs = user_git_ref.split()
    else:
        raise click.UsageError("--git-ref is required.")
    return git_refs


def _get_travis_git_refs() -> List[str]:
    git_refs = [os.getenv("TRAVIS_BRANCH", "")]
    if git_refs[0] == "":
        raise click.UsageError(
            "Using --travis but the TRAVIS_BRANCH environment variable is "
            "not detected."
        )
    return git_refs


def _get_gh_actions_git_refs() -> List[str]:
    github_ref = os.getenv("GITHUB_REF", "")
    if github_ref == "":
        raise click.UsageError(
            "Using --gh but the GITHUB_REF environment variable is "
            "not detected."
        )
    match = re.match(r"refs/(heads|tags|pull)/(?P<ref>.+)", github_ref)
    if not match:
        raise click.UsageError(
            "Could not parse the GITHUB_REF environment variable: {0}".format(
                github_ref
            )
        )
    ref = match.group("ref")
    return [ref]
